@@ -28,6 +28,7 @@
 #include <libgnome/gnome-i18n.h>
 #include <libxml/parser.h>
 #include <libxml/parserInternals.h>
+#include <libxml/xinclude.h>
 #include <libxslt/xslt.h>
 #include <libxslt/templates.h>
 #include <libxslt/transform.h>
@@ -45,7 +46,8 @@
 #define DB_STYLESHEET_PATH DATADIR"/sgml/docbook/yelp/"
 #define DB_STYLESHEET      DB_STYLESHEET_PATH"db2html.xsl"
 
-#define MAX_CHUNK_DEPTH 2
+#define BOOK_CHUNK_DEPTH 2
+#define ARTICLE_CHUNK_DEPTH 1
 
 #define d(x)
 
@@ -68,36 +70,37 @@ struct _DBWalker {
     GtkTreeIter *iter;
 
     gint depth;
+    gint max_depth;
 };
 
 static void          db_pager_class_init   (YelpDBPagerClass *klass);
 static void          db_pager_init         (YelpDBPager      *pager);
 static void          db_pager_dispose      (GObject          *gobject);
 
-void                 db_pager_error        (YelpPager   *pager);
-void                 db_pager_cancel       (YelpPager   *pager);
-void                 db_pager_finish       (YelpPager   *pager);
+void                 db_pager_error        (YelpPager        *pager);
+void                 db_pager_cancel       (YelpPager        *pager);
+void                 db_pager_finish       (YelpPager        *pager);
 
-gboolean             db_pager_process      (YelpPager   *pager);
-gchar *              db_pager_resolve_uri  (YelpPager   *pager,
-					    YelpURI     *uri);
-const GtkTreeModel * db_pager_get_sections (YelpPager   *pager);
+gboolean             db_pager_process      (YelpPager        *pager);
+gchar *              db_pager_resolve_uri  (YelpPager        *pager,
+					    YelpURI          *uri);
+const GtkTreeModel * db_pager_get_sections (YelpPager        *pager);
 
-static void     walker_walk_xml       (DBWalker         *walker);
-gboolean        walker_is_chunk       (DBWalker         *walker);
+static void          walker_walk_xml       (DBWalker         *walker);
+static gboolean      walker_is_chunk       (DBWalker         *walker);
 
-gboolean        xml_is_division       (xmlNodePtr        node);
-gboolean        xml_is_info           (xmlNodePtr        node);
-gchar *         xml_get_title         (xmlNodePtr        node);
+static gboolean      xml_is_division       (xmlNodePtr        node);
+static gboolean      xml_is_info           (xmlNodePtr        node);
+static gchar *       xml_get_title         (xmlNodePtr        node);
 
-void            xslt_yelp_document    (xsltTransformContextPtr ctxt,
-				       xmlNodePtr              node,
-				       xmlNodePtr              inst,
-				       xsltStylePreCompPtr     comp);
-void            xslt_yelp_cache       (xsltTransformContextPtr ctxt,
-				       xmlNodePtr              node,
-				       xmlNodePtr              inst,
-				       xsltStylePreCompPtr     comp);
+static void          xslt_yelp_document    (xsltTransformContextPtr ctxt,
+					    xmlNodePtr              node,
+					    xmlNodePtr              inst,
+					    xsltStylePreCompPtr     comp);
+static void          xslt_yelp_cache       (xsltTransformContextPtr ctxt,
+					    xmlNodePtr              node,
+					    xmlNodePtr              inst,
+					    xsltStylePreCompPtr     comp);
 
 static YelpPagerClass *parent_class;
 
@@ -181,7 +184,7 @@ yelp_db_pager_new (YelpURI *uri)
 {
     YelpDBPager *pager;
 
-    g_return_val_if_fail (YELP_IS_URI (uri), NULL);
+    g_return_val_if_fail (uri != NULL, NULL);
 
     pager = (YelpDBPager *) g_object_new (YELP_TYPE_DB_PAGER,
 					  "uri", uri,
@@ -196,11 +199,11 @@ yelp_db_pager_new (YelpURI *uri)
 gboolean
 db_pager_process (YelpPager *pager)
 {
-    YelpURI *uri           = yelp_pager_get_uri (pager);
-    gchar         *uri_str = yelp_uri_get_path ((YelpURI *) uri);
-    DBWalker      *walker;
-    xmlChar       *id;
-    GError        *error = NULL;
+    YelpURI   *uri = yelp_pager_get_uri (pager);
+    gchar     *path;
+    DBWalker  *walker;
+    xmlChar   *id;
+    GError    *error = NULL;
 
     xmlDocPtr               doc;
     xmlParserCtxtPtr        ctxt;
@@ -211,44 +214,56 @@ db_pager_process (YelpPager *pager)
 
     YelpDBPagerPriv *priv = YELP_DB_PAGER (pager)->priv;
 
-    gchar        *uri_slash;
     gchar        *doc_name;
     gchar        *doc_path;
-    const gchar  *params[21];
+    const gchar  *params[40];
+    gint i = 0;
 
     g_return_val_if_fail (pager != NULL, FALSE);
     g_return_val_if_fail (YELP_IS_DB_PAGER (pager), FALSE);
 
+    path = gnome_vfs_uri_to_string (uri->uri,
+				    GNOME_VFS_URI_HIDE_USER_NAME           |
+				    GNOME_VFS_URI_HIDE_PASSWORD            |
+				    GNOME_VFS_URI_HIDE_HOST_NAME           |
+				    GNOME_VFS_URI_HIDE_HOST_PORT           |
+				    GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD     |
+				    GNOME_VFS_URI_HIDE_FRAGMENT_IDENTIFIER );
+
     g_object_ref (pager);
 
     yelp_toc_pager_pause (yelp_toc_pager_get ());
 
     ctxt = xmlNewParserCtxt ();
     doc = xmlCtxtReadFile (ctxt,
-			   (const char *) uri_str,
+			   (const char *) path,
 			   NULL,
 			   XML_PARSE_DTDLOAD  |
-			   XML_PARSE_XINCLUDE |
 			   XML_PARSE_NOCDATA  |
 			   XML_PARSE_NOENT    |
 			   XML_PARSE_NONET    );
 
     if (doc == NULL) {
-	gchar *str_uri = yelp_uri_to_string (uri);
-	g_set_error (&error,
-		     YELP_ERROR,
-		     YELP_ERROR_FAILED_OPEN,
-		     _("The document '%s' could not be opened"), str_uri);
+	yelp_set_error (&error, YELP_ERROR_NO_DOC);
 	yelp_pager_error (pager, error);
-
-	g_free (str_uri);
 	return FALSE;
     }
 
+    xmlXIncludeProcessFlags (doc,
+			     XML_PARSE_DTDLOAD  |
+			     XML_PARSE_NOCDATA  |
+			     XML_PARSE_NOENT    |
+			     XML_PARSE_NONET    );
+
     walker = g_new0 (DBWalker, 1);
-    walker->pager = YELP_DB_PAGER (pager);
-    walker->doc   = doc;
-    walker->cur   = xmlDocGetRootElement (walker->doc);
+    walker->pager     = YELP_DB_PAGER (pager);
+    walker->doc       = doc;
+    walker->cur       = xmlDocGetRootElement (walker->doc);
+
+    if (!xmlStrcmp (xmlDocGetRootElement (doc)->name, BAD_CAST "book"))
+	walker->max_depth = BOOK_CHUNK_DEPTH;
+    else
+	walker->max_depth = ARTICLE_CHUNK_DEPTH;
 
     id = xmlGetProp (walker->cur, "id");
     if (id)
@@ -267,34 +282,30 @@ db_pager_process (YelpPager *pager)
     while (gtk_events_pending ())
 	gtk_main_iteration ();
 
-    uri_slash = g_strrstr (uri_str, "/");
-    doc_name  = g_strdup  (uri_slash + 1);
-    doc_path  = g_strndup (uri_str, uri_slash - uri_str + 1);
-
+    doc_name = gnome_vfs_uri_extract_short_name (uri->uri);
+    doc_path = gnome_vfs_uri_extract_dirname (uri->uri);
     p_doc_name = g_strconcat("\"", doc_name, "\"", NULL);
     p_doc_path = g_strconcat("\"file://", doc_path, "/\"", NULL);
 
-    params[0]  = "doc_name";
-    params[1]  = p_doc_name;
-    params[2]  = "doc_path";
-    params[3]  = p_doc_path;
-    params[4]  = "stylesheet_path";
-    params[5]  = "\"file://" DB_STYLESHEET_PATH "/\"";
-    params[6]  = "chunk_depth";
-    params[7]  = "2";
-    params[8]  = "html_extension";
-    params[9]  = "\"\"";
-    params[10] = "resolve_xref_chunk";
-    params[11] = "0";
-    params[12] = "mediaobject_path";
-    params[13] = p_doc_path;
-    params[14] = "color_gray_background";
-    params[15] = yelp_theme_get_gray_background ();
-    params[16] = "color_gray_border";
-    params[17] = yelp_theme_get_gray_border ();
-    params[18] = "admon_graphics_path";
-    params[19] = "\"file://" DATADIR "/yelp/icons/\"";
-    params[20] = NULL;
+    params[i++] = "doc_name";
+    params[i++] = p_doc_name;
+    params[i++] = "doc_path";
+    params[i++] = p_doc_path;
+    params[i++] = "stylesheet_path";
+    params[i++] = "\"file://" DB_STYLESHEET_PATH "/\"";
+    params[i++] = "html_extension";
+    params[i++] = "\"\"";
+    params[i++] = "resolve_xref_chunk";
+    params[i++] = "0";
+    params[i++] = "mediaobject_path";
+    params[i++] = p_doc_path;
+    params[i++] = "color_gray_background";
+    params[i++] = yelp_theme_get_gray_background ();
+    params[i++] = "color_gray_border";
+    params[i++] = yelp_theme_get_gray_border ();
+    params[i++] = "admon_graphics_path";
+    params[i++] = "\"file://" DATADIR "/yelp/icons/\"";
+    params[i++] = NULL;
 
     stylesheet = xsltParseStylesheetFile (DB_STYLESHEET);
     tctxt      = xsltNewTransformContext (stylesheet,
@@ -320,12 +331,13 @@ db_pager_process (YelpPager *pager)
 
     g_free (p_doc_name);
     g_free (p_doc_path);
+    g_free (doc_name);
+    g_free (doc_path);
+
     xmlFreeDoc (doc);
     xsltFreeStylesheet (stylesheet);
     xmlFreeParserCtxt (ctxt);
 
-    g_free (doc_name);
-    g_free (doc_path);
 
     yelp_pager_set_state (pager, YELP_PAGER_STATE_FINISHED);
     g_signal_emit_by_name (pager, "finish");
@@ -357,16 +369,16 @@ db_pager_finish (YelpPager   *pager)
 gchar *
 db_pager_resolve_uri (YelpPager *pager, YelpURI *uri)
 {
-    YelpDBPager *db_pager;
-    gchar       *frag_id;
-    gchar       *page_id = NULL;
+    YelpDBPager  *db_pager;
+    const gchar  *frag_id;
+    gchar        *page_id = NULL;
 
     g_return_val_if_fail (pager != NULL, NULL);
     g_return_val_if_fail (YELP_IS_DB_PAGER (pager), NULL);
 
     db_pager = YELP_DB_PAGER (pager);
 
-    frag_id = yelp_uri_get_fragment (uri);
+    frag_id = gnome_vfs_uri_get_fragment_identifier (uri->uri);
 
     if (frag_id)
 	page_id = g_hash_table_lookup (db_pager->priv->frags_hash,
@@ -376,7 +388,6 @@ db_pager_resolve_uri (YelpPager *pager, YelpURI *uri)
     else
 	page_id = "index";
 
-    g_free (frag_id);
     return g_strdup (page_id);
 }
 
@@ -519,6 +530,19 @@ walker_walk_xml (DBWalker *walker)
     GtkTreeIter *old_iter;
     YelpDBPagerPriv *priv = walker->pager->priv;
 
+    if (walker->depth == 0) {
+	cur = walker->cur;
+	for ( ; cur; cur = cur->prev)
+	    if (cur->type == XML_PI_NODE)
+		if (!xmlStrcmp (cur->name,
+				(const xmlChar *) "yelp:chunk-depth")) {
+		    gint max = atoi (cur->content);
+		    if (max)
+			walker->max_depth = max;
+		    break;
+		}
+    }
+
     id = xmlGetProp (walker->cur, "id");
     if (!id && walker->cur->parent->type == XML_DOCUMENT_NODE) {
 	id = g_strdup ("index");
@@ -596,6 +620,7 @@ gchar *
 xml_get_title (xmlNodePtr node)
 {
     gchar *title = NULL;
+    gchar *ret   = NULL;
     xmlNodePtr cur;
 
     if (xml_is_info (node))
@@ -621,13 +646,17 @@ xml_get_title (xmlNodePtr node)
     if (!title)
 	title = g_strdup (_("Unknown"));
 
-    return title;
+    // This really isn't adequate for what we want.
+    ret = g_strdup (g_strstrip (title));
+    g_free (title);
+
+    return ret;
 }
 
 gboolean
 walker_is_chunk (DBWalker *walker)
 {
-    if (walker->depth <= MAX_CHUNK_DEPTH) {
+    if (walker->depth <= walker->max_depth) {
 	if (xml_is_division (walker->cur))
 	    return TRUE;
 	else if (walker->depth == 1 && xml_is_info (walker->cur))
